#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "trap.h"

void
proc_result_free(proc_result *result)
{
	if (!result)
		return;
	free(result->host);
	free(result->service);
	free(result->tag);
	free(result->message);
	free(result);
	result = NULL;
}

void
proc_handler_free(proc_handler *handler)
{
	free(handler->name);
	free(handler->script);
	free(handler->context);
	if (handler->next)
		proc_handler_free(handler->next);
	free(handler);
}

void
proc_field_free(proc_field *field)
{
	if (!field)
		return;
	free(field->oid);
	if (field->type == FIELD_STRING)
		free(field->value.string);
	if (field->next) {
		proc_field_free(field->next);
	}
	free(field);
	field = NULL;
}

void
proc_trap_free(proc_trap *trap)
{
	if (!trap)
		return;
	free(trap->host);
	free(trap->oid);
	if (trap->fields)
		proc_field_free(trap->fields);
	free(trap);
}

char *
display_fields(const proc_field *field)
{
	char *tmp, *buf = strdup("");
	for (; field; field = field->next)
	{
		switch (field->type) {
			case FIELD_STRING:
				if (asprintf(&tmp, "%s%s = %s\n", buf, field->oid, field->value.string) < 0)
					return NULL;
				free(buf);
				buf = tmp;
				break;
			case FIELD_NUMERIC:
				if (asprintf(&tmp, "%s%s = %lld\n", buf, field->oid, field->value.number) < 0)
					return NULL;
				free(buf);
				buf = tmp;
				break;
			case FIELD_BOOLEAN:
				if (asprintf(&tmp, "%s%s = %s\n", buf, field->oid, field->value.boolean ? "TRUE" : "FALSE") < 0)
					return NULL;
				free(buf);
				buf = tmp;
				break;
			case FIELD_NULL:
				if (asprintf(&tmp, "%s%s = NULL\n", buf, field->oid) < 0)
					return NULL;
				free(buf);
				buf = tmp;
				break;
			case FIELD_TIME:
				if (asprintf(&tmp, "%s%s = @%ld\n", buf, field->oid, field->value.time) < 0)
					return NULL;
				free(buf);
				buf = tmp;
				break;
		}
	}
	return buf;
}

char *
display_trap(const proc_trap *trap)
{
	if (!trap)
		return NULL;
	char *buf;
	char *fields = display_fields(trap->fields);
	if (asprintf(&buf, "%s -> %s @ %ld (%d):\n%s", trap->host, trap->oid, trap->timestamp, trap->id, fields) < 0)
		return NULL;
	free(fields);
	return buf;
}
